@@ -64,7 +64,7 @@ import (
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/clientcmd"
 	"k8s.io/client-go/tools/remotecommand"
-	inferencev1alpha2 "sigs.k8s.io/gateway-api-inference-extension/api/v1alpha2"
+	inferencev1 "sigs.k8s.io/gateway-api-inference-extension/api/v1"
 	gatewayapiinferenceclient "sigs.k8s.io/gateway-api-inference-extension/client-go/clientset/versioned"
 	gatewayapiinferencefake "sigs.k8s.io/gateway-api-inference-extension/client-go/clientset/versioned/fake"
 	gatewayapiv1 "sigs.k8s.io/gateway-api/apis/v1"
@@ -170,9 +170,6 @@ type CLIClient interface {
 	// Revision of the Istio control plane.
 	Revision() string
 
-	// EnvoyDo makes a http request to the Envoy in the specified pod.
-	EnvoyDo(ctx context.Context, podName, podNamespace, method, path string) ([]byte, error)
-
 	// EnvoyDoWithPort makes a http request to the Envoy in the specified pod and port.
 	EnvoyDoWithPort(ctx context.Context, podName, podNamespace, method, path string, port int) ([]byte, error)
 
@@ -260,7 +257,7 @@ func setupFakeClient[T fakeClient](fc T, group string, objects []runtime.Object)
 		if strings.Contains(g, "istio.io") {
 			return group == "istio"
 		}
-		if strings.Contains(g, "inference.networking.x-k8s.io") {
+		if strings.Contains(g, inferencev1.GroupVersion.Group) {
 			return group == "inference"
 		}
 		if strings.Contains(g, "gateway.networking.k8s.io") {
@@ -296,6 +293,76 @@ func setupFakeClient[T fakeClient](fc T, group string, objects []runtime.Object)
 	return fc
 }
 
+// NewErroringFakeClient creates a new fake client that always returns errors
+// on lists and watches to simulate errors from the API server.
+func NewErroringFakeClient(objects ...runtime.Object) CLIClient {
+	c := &client{
+		informerWatchesPending: atomic.NewInt32(0),
+		clusterID:              "fake",
+	}
+
+	c.kube = setupFakeClient(fake.NewClientset(), "kube", objects)
+
+	c.config = &rest.Config{
+		Host: "server",
+	}
+
+	c.informerFactory = informerfactory.NewSharedInformerFactory()
+	s := FakeIstioScheme
+
+	c.metadata = metadatafake.NewSimpleMetadataClient(s)
+	c.dynamic = dynamicfake.NewSimpleDynamicClient(s)
+	c.istio = setupFakeClient(istiofake.NewSimpleClientset(), "istio", objects)
+	c.gatewayapi = setupFakeClient(gatewayapifake.NewSimpleClientset(), "gateway", objects)
+	c.gatewayapiinference = setupFakeClient(gatewayapiinferencefake.NewSimpleClientset(), "inference", objects)
+	c.extSet = extfake.NewClientset()
+
+	listReactor := func(action clienttesting.Action) (handled bool, ret runtime.Object, err error) {
+		return true, nil, errors.New("fake client list error")
+	}
+	watchReactor := func(tracker clienttesting.ObjectTracker) func(action clienttesting.Action) (handled bool, ret watch.Interface, err error) {
+		return func(action clienttesting.Action) (handled bool, ret watch.Interface, err error) {
+			return true, nil, errors.New("fake client watch error")
+		}
+	}
+	// https://github.com/kubernetes/client-go/issues/439
+	createReactor := func(action clienttesting.Action) (handled bool, ret runtime.Object, err error) {
+		ret = action.(clienttesting.CreateAction).GetObject()
+		meta, ok := ret.(metav1.Object)
+		if !ok {
+			return handled, ret, err
+		}
+
+		if meta.GetName() == "" && meta.GetGenerateName() != "" {
+			meta.SetName(names.SimpleNameGenerator.GenerateName(meta.GetGenerateName()))
+		}
+
+		return handled, ret, err
+	}
+	for _, fc := range []fakeClient{
+		c.kube.(*fake.Clientset),
+		c.istio.(*istiofake.Clientset),
+		c.gatewayapi.(*gatewayapifake.Clientset),
+		c.gatewayapiinference.(*gatewayapiinferencefake.Clientset),
+		c.dynamic.(*dynamicfake.FakeDynamicClient),
+		c.metadata.(*metadatafake.FakeMetadataClient),
+	} {
+		fc.PrependReactor("list", "*", listReactor)
+		fc.PrependWatchReactor("*", watchReactor(fc.Tracker()))
+		fc.PrependReactor("create", "*", createReactor)
+	}
+
+	c.fastSync = true
+
+	c.version = lazy.NewWithRetry(c.kube.Discovery().ServerVersion)
+
+	if NewCrdWatcher != nil {
+		c.crdWatcher = NewCrdWatcher(c)
+	}
+
+	return c
+}
+
 // NewFakeClient creates a new, fake, client
 func NewFakeClient(objects ...runtime.Object) CLIClient {
 	c := &client{
@@ -348,14 +415,14 @@ func NewFakeClient(objects ...runtime.Object) CLIClient {
 		ret = action.(clienttesting.CreateAction).GetObject()
 		meta, ok := ret.(metav1.Object)
 		if !ok {
-			return
+			return handled, ret, err
 		}
 
 		if meta.GetName() == "" && meta.GetGenerateName() != "" {
 			meta.SetName(names.SimpleNameGenerator.GenerateName(meta.GetGenerateName()))
 		}
 
-		return
+		return handled, ret, err
 	}
 	for _, fc := range []fakeClient{
 		c.kube.(*fake.Clientset),
@@ -848,7 +915,7 @@ func (c *client) PodExecCommands(podName, podNamespace, container string, comman
 
 	stdout = stdoutBuf.String()
 	stderr = stderrBuf.String()
-	return
+	return stdout, stderr, err
 }
 
 func (c *client) PodExec(podName, podNamespace, container string, command string) (stdout, stderr string, err error) {
@@ -925,10 +992,6 @@ func (c *client) AllDiscoveryDo(ctx context.Context, istiodNamespace, path strin
 	return nil, nil
 }
 
-func (c *client) EnvoyDo(ctx context.Context, podName, podNamespace, method, path string) ([]byte, error) {
-	return c.portForwardRequest(ctx, podName, podNamespace, method, path, 15000)
-}
-
 func (c *client) EnvoyDoWithPort(ctx context.Context, podName, podNamespace, method, path string, port int) ([]byte, error) {
 	return c.portForwardRequest(ctx, podName, podNamespace, method, path, port)
 }
@@ -1385,7 +1448,7 @@ func istioScheme() *runtime.Scheme {
 	utilruntime.Must(gatewayapibeta.Install(scheme))
 	utilruntime.Must(gatewayapiv1.Install(scheme))
 	utilruntime.Must(gatewayx.Install(scheme))
-	utilruntime.Must(inferencev1alpha2.Install(scheme))
+	utilruntime.Must(inferencev1.Install(scheme))
 	utilruntime.Must(apiextensionsv1.AddToScheme(scheme))
 	return scheme
 }
