@@ -34,6 +34,7 @@ import (
 	"k8s.io/client-go/tools/clientcmd"
 	"k8s.io/client-go/tools/clientcmd/api"
 
+	"istio.io/api/annotation"
 	"istio.io/api/label"
 	"istio.io/istio/pilot/pkg/features"
 	"istio.io/istio/pkg/util/sets"
@@ -392,6 +393,8 @@ func StripPodUnusedFields(obj any) (any, error) {
 	}
 	// ManagedFields is large and we never use it
 	t.GetObjectMeta().SetManagedFields(nil)
+	// Proxy overrides are never used in the cache and can be very big
+	delete(t.GetObjectMeta().GetAnnotations(), annotation.ProxyOverrides.Name)
 	// only container ports can be used
 	if pod := obj.(*corev1.Pod); pod != nil {
 		containers := []corev1.Container{}
